Package: diseasystore
Title: Feature Stores for the 'diseasy' Framework
Version: 0.3.3
Authors@R: 
    c(person("Rasmus Skytte", "Randl\u00F8v", , "rske@ssi.dk", role = c("aut", "cre"),
             comment = c(ORCID = "0000-0002-5860-3838")),
      person("Kaare", "Gr\u00E6sb\u00F8ll", , "kagr@ssi.dk", role = c("rev"),
             comment = c(ORCID = "0000-0002-6258-8212")),
      person("Kasper", "Schou Telkamp", , , role = c("rev"),
             comment = c(ORCID = "0009-0001-5126-0190")),
      person("Lasse Engbo", "Christiansen", , "lsec@ssi.dk", role = c("rev"),
             comment = c(ORCID = "0000-0001-5019-1931")),
      person("Marcus Munch", "Gr\u00FCnewald", , , role = c("rev"),
             comment = c(ORCID = "0009-0006-8090-406X")),
      person("Sofia Myrup", "Otero", , "smot@ssi.dk", role = c("rev")),
      person("Statens Serum Institut, SSI", role = c("cph", "fnd")))
Description: 
  Simple feature stores and tools for creating personalised feature stores.
  'diseasystore' powers feature stores which can automatically link and aggregate features to a given stratification
  level. These feature stores are automatically time-versioned (powered by the 'SCDB' package) and allows you to easily
  and dynamically compute features as part of your continuous integration.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
Language: en-GB
Depends: R (>= 4.3.0)
Imports: checkmate, curl, DBI, dbplyr, dplyr, glue, ISOweek, jsonlite,
        lubridate, pkgcond, purrr, readr, rlang, R6, SCDB (>= 0.5.2),
        stringr, tidyr, tidyselect
Suggests: devtools, duckdb, ggplot2, here, knitr, lintr,
        microbenchmark, odbc (>= 1.6.4), pkgdown, rmarkdown, RSQLite,
        RPostgres, testthat (>= 3.0.0), tibble, spelling, usethis,
        withr
VignetteBuilder: knitr
URL: https://github.com/ssi-dk/diseasystore,
        https://ssi-dk.github.io/diseasystore/
BugReports: https://github.com/ssi-dk/diseasystore/issues
Config/testthat/edition: 3
LazyData: true
NeedsCompilation: no
Packaged: 2026-03-20 12:04:54 UTC; B246705
Author: Rasmus Skytte Randløv [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5860-3838>),
  Kaare Græsbøll [rev] (ORCID: <https://orcid.org/0000-0002-6258-8212>),
  Kasper Schou Telkamp [rev] (ORCID:
    <https://orcid.org/0009-0001-5126-0190>),
  Lasse Engbo Christiansen [rev] (ORCID:
    <https://orcid.org/0000-0001-5019-1931>),
  Marcus Munch Grünewald [rev] (ORCID:
    <https://orcid.org/0009-0006-8090-406X>),
  Sofia Myrup Otero [rev],
  Statens Serum Institut, SSI [cph, fnd]
Maintainer: Rasmus Skytte Randløv <rske@ssi.dk>
Repository: CRAN
Date/Publication: 2026-03-20 13:20:02 UTC
Built: R 4.6.0; ; 2026-04-26 02:02:38 UTC; unix
