# Generated by roxygen2: do not edit by hand

S3method(add_years,"Microsoft SQL Server")
S3method(add_years,PqConnection)
S3method(add_years,SQLiteConnection)
S3method(add_years,duckdb_connection)
S3method(age_on_date,"Microsoft SQL Server")
S3method(age_on_date,PqConnection)
S3method(age_on_date,SQLiteConnection)
S3method(age_on_date,duckdb_connection)
export("%.%")
export(DiseasystoreBase)
export(DiseasystoreEcdcRespiratoryViruses)
export(DiseasystoreGoogleCovid19)
export(DiseasystoreSimulist)
export(FeatureHandler)
export(add_years)
export(age_labels)
export(age_on_date)
export(available_diseasystores)
export(diseasyoption)
export(diseasystore_exists)
export(drop_diseasystore)
export(get_diseasystore)
export(key_join_count)
export(key_join_max)
export(key_join_min)
export(key_join_sum)
export(source_conn_github)
export(source_conn_path)
export(test_diseasystore)
export(to_diseasystore_case)
importFrom(ISOweek,ISOweek2date)
importFrom(R6,R6Class)
importFrom(curl,has_internet)
importFrom(dbplyr,window_order)
importFrom(rlang,.data)
